/*
    Copyright (C) 2002 Kjetil S. Matheussen / Notam.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#include <stdlib.h>
#include <stdio.h>
#include <sched.h>
#include <sys/mman.h>

#include <vstserver.h>

#include "s_.h"

#include "processthread.h"


/* Should have had a watchdog here... */

static void set_realtime(struct VSTClient *client){
    struct sched_param par;
    par.sched_priority = (sched_get_priority_max(SCHED_FIFO) + sched_get_priority_min(SCHED_FIFO) ) / 2;

    if(sched_setscheduler(0,SCHED_FIFO,&par)==-1){
      fprintf(stderr,"VSTSERVANT/PROCESS_thread: Unable to set SCHED_FIFO realtime priority for \"%s\".\n",client->name);
    }else{
      printf("VSTSERVANT/PROCESS_thread: Realtime priority set for \"%s\".\n",client->name);
    }

    if(mlockall(MCL_FUTURE)==-1){
    	fprintf(stderr,"VSTSERVANT/PROCESS_thread: Unable to lock memory for \"%s\".\n",client->name);
    }
}


void *PROCESS_thread(void *arg){
  struct VSTClient *client=(struct VSTClient *)arg;
  char name[500];
  
  while(C_acceptIncomingConnection(client->process_input)==false);

  sprintf(name,"%s",client->name);

  if(client->realtime==1){
    system("/usr/bin/givertcap");
    set_realtime(client);
  }
  
  client->process_thread_status=1;

  for(;;){
    int sampleframes;
    int num_read=C_receive(client->process_input,&sampleframes,sizeof(int));
    if(num_read!=sizeof(int)){
      if(num_read==0){
	fprintf(stderr,"VSTSERVANT/PROCESS_thread: Seems like client has died. Exiting \"%s\".\n",name);
	return NULL;
      }else{
	fprintf(stderr,"VSTSERVANT/PROCESS_thread: num_read not %d but %d for \"%s\".\n",sizeof(int),num_read,name);
      }
      continue;
    }

    if(sampleframes==0){
      printf("VSTSERVANT/PROCESS_thread: \"%s\" received delete.\n",name);
      return NULL;
    }

    if(sampleframes>0){
      VSTS_process(client,sampleframes);
    }else{
      VSTS_processReplacing(client,-sampleframes);
    }
  }

  return NULL;
}

